/**
 * Informatique Industrielle et Java
 * exercice 5.2 du chapitre 5
 *
 * Projet: gestion de conflits sur un port de communication
 *
 * Cette classe gre les conflits sur un port, elle permet d'ouvrir
 * un port donn, de le fermer et s'enregistre pour recevoir les 
 * demandes d'accs qui proviennent d'autres applications
 *
 * @author F.Mallet
 * @version 2003.04
 */
import javax.comm.CommPortOwnershipListener;
import javax.comm.PortInUseException;
import javax.comm.CommPort;
import javax.comm.CommPortIdentifier;
import javax.swing.JOptionPane;

public class GestionnaireConflits implements CommPortOwnershipListener {
	//identificateur du port surlequel on opre
	private CommPortIdentifier ident;
	
	// nom de l'application qui demande la gestion de conflits
	private String nomAppli;
	
	// port manipul : cette classe n'a pas besoin de savoir si c'est un port srie ou parallle
	private CommPort port = null;

	/**
	 * @param ident reprsente l'identificateur du port surlequel on opre
	 * @param nomAppli est le nom de l'application qui demande la gestion de conflits
	 */
	public GestionnaireConflits(CommPortIdentifier ident, String nomAppli) {
		this.ident = ident;
		this.nomAppli = nomAppli;
	}
	
	/**
	 * ouvre le port dont on est responsable
	 * @return port s'il est ouvert avec succes, null sinon
	 */
	public CommPort ouvrePort() {
		// On essaye d'ouvrir le port et on attend 5 secondes au maximum qu'il se libre
		try {
		  port = ident.open(nomAppli, 5000);
		  ident.addPortOwnershipListener(this);
		} catch (PortInUseException piue) {
			System.err.println("Le port "+ident.getName()+" est dj utilis");
		} finally {
			return port;
		}
	}

	// ferme le port dont on est responsable
	public void fermePort() {
		if (port != null) {
			port.close();
			port = null;
		}
	}
	
	// L'interface PortOwnershipListener declare la mthode ownershipChange(int)
	public void ownershipChange(int val) {
		switch(val) {
		case PORT_OWNERSHIP_REQUESTED:	
			int res = JOptionPane.showConfirmDialog(null, 
				"Voulez-vous cder le port "+ident.getName()+" ?",
				nomAppli+": conflit d'acces", 
				JOptionPane.YES_NO_OPTION);
			
			if (res == JOptionPane.YES_OPTION) {
				fermePort();
			}
			break;
		case PORT_OWNED:
			break;
		case PORT_UNOWNED:
			ident.removePortOwnershipListener(this);
		default:
			break;
		} 
	}
	
	/** @return reference sur le port ouvert */
	public CommPort getPort() {
		return port;
	}
	
	// La mthode toString est redfinie pour prsenter une information
	// prcise sur le port dont on est responsable
	public String toString() {
		String res = ident.getName();
		if (ident.getPortType() == CommPortIdentifier.PORT_SERIAL) {
			res += " srie ";
		} else {
			res += " parallle ";
		}
		
		if (ident.isCurrentlyOwned()) {
			res += "("+ident.getCurrentOwner()+")";
		}
		
		return res;
	}
}
